@@ -15,7 +15,7 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- * 
+ *
  * Authors: Xavier Claessens <xclaesse@gmail.com>
  */
 
@@ -41,7 +41,8 @@ typedef enum {
 GtkWidget * empathy_contact_menu_new           (EmpathyContact             *contact,
 						EmpathyContactFeatureFlags  features);
 GtkWidget * empathy_contact_chat_menu_item_new (EmpathyContact             *contact);
-GtkWidget * empathy_contact_call_menu_item_new (EmpathyContact             *contact);
+GtkWidget * empathy_contact_audio_call_menu_item_new (EmpathyContact             *contact);
+GtkWidget * empathy_contact_video_call_menu_item_new (EmpathyContact             *contact);
 GtkWidget * empathy_contact_log_menu_item_new  (EmpathyContact             *contact);
 GtkWidget * empathy_contact_info_menu_item_new (EmpathyContact             *contact);
 GtkWidget * empathy_contact_edit_menu_item_new (EmpathyContact             *contact);
