@@ -15,15 +15,15 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- * 
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ *
  * Authors: Mikael Hallendal <micke@imendio.com>
  *          Richard Hult <richard@imendio.com>
  *          Martyn Russell <martyn@imendio.com>
  *          Xavier Claessens <xclaesse@gmail.com>
  *          Jonny Lamb <jonny.lamb@collabora.co.uk>
- * 
+ *
  *          Part of this file is copied from GtkSourceView (gtksourceiter.c):
  *          Paolo Maggi
  *          Jeroen Zwartepoorte
@@ -37,7 +37,6 @@
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 #include <gio/gio.h>
-#include <glade/glade.h>
 #include <canberra-gtk.h>
 
 #include <libmissioncontrol/mc-profile.h>
@@ -89,39 +88,33 @@ empathy_uri_regex_dup_singleton (void)
 	return g_regex_ref (uri_regex);
 }
 
-struct SizeData {
-	gint     width;
-	gint     height;
-	gboolean preserve_aspect_ratio;
-};
-
-static GladeXML *
-get_glade_file (const gchar *filename,
-		const gchar *root,
-		const gchar *domain,
-		const gchar *first_required_widget,
-		va_list      args)
+static GtkBuilder *
+builder_get_file_valist (const gchar *filename,
+			 const gchar *first_object,
+			 va_list      args)
 {
-	GladeXML   *gui;
-	const char *name;
-	GtkWidget **widget_ptr;
+	GtkBuilder  *gui;
+	const gchar *name;
+	GObject    **object_ptr;
+	GError      *error = NULL;
 
-	DEBUG ("Loading glade file %s", filename);
+	DEBUG ("Loading file %s", filename);
 
-	gui = glade_xml_new (filename, root, domain);
-
-	if (!gui) {
-		g_warning ("Couldn't find necessary glade file '%s'", filename);
+	gui = gtk_builder_new ();
+	if (!gtk_builder_add_from_file (gui, filename, &error)) {
+		g_critical ("GtkBuilder Error: %s", error->message);
+		g_clear_error (&error);
+		g_object_unref (gui);
+		return NULL;
 	}
 
-	for (name = first_required_widget; name; name = va_arg (args, char *)) {
-		widget_ptr = va_arg (args, void *);
+	for (name = first_object; name; name = va_arg (args, const gchar *)) {
+		object_ptr = va_arg (args, GObject**);
 
-		*widget_ptr = glade_xml_get_widget (gui, name);
+		*object_ptr = gtk_builder_get_object (gui, name);
 
-		if (!*widget_ptr) {
-			g_warning ("Glade file '%s' is missing widget '%s'.",
-				   filename, name);
+		if (!*object_ptr) {
+			g_warning ("File is missing object '%s'.", name);
 			continue;
 		}
 	}
@@ -129,116 +122,65 @@ get_glade_file (const gchar *filename,
 	return gui;
 }
 
-void
-empathy_glade_get_file_simple (const gchar *filename,
-			      const gchar *root,
-			      const gchar *domain,
-			      const gchar *first_required_widget, ...)
+GtkBuilder *
+empathy_builder_get_file (const gchar *filename,
+			  const gchar *first_object,
+			  ...)
 {
-	va_list   args;
-	GladeXML *gui;
-
-	va_start (args, first_required_widget);
-
-	gui = get_glade_file (filename,
-			      root,
-			      domain,
-			      first_required_widget,
-			      args);
+	GtkBuilder *gui;
+	va_list     args;
 
+	va_start (args, first_object);
+	gui = builder_get_file_valist (filename, first_object, args);
 	va_end (args);
 
-	if (gui) {
-		g_object_unref (gui);
-	}
-}
-
-GladeXML *
-empathy_glade_get_file (const gchar *filename,
-		       const gchar *root,
-		       const gchar *domain,
-		       const gchar *first_required_widget, ...)
-{
-	va_list   args;
-	GladeXML *gui;
-
-	va_start (args, first_required_widget);
-
-	gui = get_glade_file (filename,
-			      root,
-			      domain,
-			      first_required_widget,
-			      args);
-
-	va_end (args);
-
-	if (!gui) {
-		return NULL;
-	}
-
 	return gui;
 }
 
 void
-empathy_glade_connect (GladeXML *gui,
-		      gpointer  user_data,
-		      gchar     *first_widget, ...)
+empathy_builder_connect (GtkBuilder *gui,
+			 gpointer    user_data,
+			 gchar      *first_object,
+			 ...)
 {
 	va_list      args;
 	const gchar *name;
 	const gchar *signal;
-	GtkWidget   *widget;
-	gpointer    *callback;
+	GObject     *object;
+	GCallback    callback;
 
-	va_start (args, first_widget);
+	va_start (args, first_object);
+	for (name = first_object; name; name = va_arg (args, const gchar *)) {
+		signal = va_arg (args, const gchar *);
+		callback = va_arg (args, GCallback);
 
-	for (name = first_widget; name; name = va_arg (args, char *)) {
-		signal = va_arg (args, void *);
-		callback = va_arg (args, void *);
-
-		widget = glade_xml_get_widget (gui, name);
-		if (!widget) {
-			g_warning ("Glade file is missing widget '%s', aborting",
-				   name);
+		object = gtk_builder_get_object (gui, name);
+		if (!object) {
+			g_warning ("File is missing object '%s'.", name);
 			continue;
 		}
 
-		g_signal_connect (widget,
-				  signal,
-				  G_CALLBACK (callback),
-				  user_data);
+		g_signal_connect (object, signal, callback, user_data);
 	}
 
 	va_end (args);
 }
 
-void
-empathy_glade_setup_size_group (GladeXML         *gui,
-			       GtkSizeGroupMode  mode,
-			       gchar            *first_widget, ...)
+GtkWidget *
+empathy_builder_unref_and_keep_widget (GtkBuilder *gui,
+				       GtkWidget  *widget)
 {
-	va_list       args;
-	GtkWidget    *widget;
-	GtkSizeGroup *size_group;
-	const gchar  *name;
+	/* On construction gui sinks the initial reference to widget. When gui
+	 * is finalized it will drop its ref to widget. We take our own ref to
+	 * prevent widget being finalised. The widget is forced to have a
+	 * floating reference, like when it was initially unowned so that it can
+	 * be used like any other GtkWidget. */
 
-	va_start (args, first_widget);
+	g_object_ref (widget);
+	g_object_force_floating (G_OBJECT (widget));
+	g_object_unref (gui);
 
-	size_group = gtk_size_group_new (mode);
-
-	for (name = first_widget; name; name = va_arg (args, char *)) {
-		widget = glade_xml_get_widget (gui, name);
-		if (!widget) {
-			g_warning ("Glade file is missing widget '%s'", name);
-			continue;
-		}
-
-		gtk_size_group_add_widget (size_group, widget);
-	}
-
-	g_object_unref (size_group);
-
-	va_end (args);
+	return widget;
 }
 
 const gchar *
@@ -346,6 +288,12 @@ out:
 	return pixbuf;
 }
 
+struct SizeData {
+	gint     width;
+	gint     height;
+	gboolean preserve_aspect_ratio;
+};
+
 static void
 pixbuf_from_avatar_size_prepared_cb (GdkPixbufLoader *loader,
 				     int              width,
@@ -661,7 +609,7 @@ g_utf8_strcasestr (const gchar *haystack, const gchar *needle)
 
 	if (needle_len == 0)
 	{
-		ret = (gchar *)haystack;
+		ret = (gchar *) haystack;
 		goto finally_1;
 	}
 
@@ -671,7 +619,7 @@ g_utf8_strcasestr (const gchar *haystack, const gchar *needle)
 		goto finally_1;
 	}
 
-	p = (gchar*)caseless_haystack;
+	p = (gchar *) caseless_haystack;
 	needle_len = strlen (needle);
 	i = 0;
 
@@ -767,7 +715,7 @@ forward_chars_with_skipping (GtkTextIter *iter,
 			/* being UTF8 correct sucks; this accounts for extra
 			   offsets coming from canonical decompositions of
 			   UTF8 characters (e.g. accented characters) which
-			   g_utf8_normalize() performs */
+			   g_utf8_normalize () performs */
 			gchar *normal;
 			gchar buffer[6];
 			gint buffer_len;
@@ -1020,7 +968,7 @@ empathy_text_iter_forward_search (const GtkTextIter   *iter,
 		}
 	} while (gtk_text_iter_forward_line (&search));
 
-	g_strfreev ((gchar**)lines);
+	g_strfreev ((gchar **) lines);
 
 	return retval;
 }
@@ -1048,7 +996,7 @@ g_utf8_strrcasestr (const gchar *haystack, const gchar *needle)
 
 	if (needle_len == 0)
 	{
-		ret = (gchar *)haystack;
+		ret = (gchar *) haystack;
 		goto finally_1;
 	}
 
@@ -1266,7 +1214,7 @@ empathy_text_iter_backward_search (const GtkTextIter   *iter,
 		}
 	}
 
-	g_strfreev ((gchar**)lines);
+	g_strfreev ((gchar **) lines);
 
 	return retval;
 }
@@ -1292,7 +1240,7 @@ empathy_window_get_is_visible (GtkWindow *window)
 	return TRUE;
 }
 
-void 
+void
 empathy_window_iconify (GtkWindow *window, GtkStatusIcon *status_icon)
 {
 	GdkRectangle  icon_location;
@@ -1402,7 +1350,7 @@ empathy_url_show (GtkWidget *parent,
 		GtkWidget *dialog;
 
 		dialog = gtk_message_dialog_new (NULL, 0,
-						 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, 
+						 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
 						 _("Unable to open URI"));
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 							  "%s", error->message);
@@ -1459,7 +1407,7 @@ static void
 file_manager_send_file_request_cb (EmpathyDispatchOperation *operation,
 				   const GError *error, gpointer user_data)
 {
-	GFile *file = (GFile *)user_data;
+	GFile *file = (GFile *) user_data;
 	EmpathyTpFile *tp_file;
 
 	if (error != NULL) {
@@ -1676,7 +1624,7 @@ empathy_sound_play_full (GtkWidget *widget, EmpathySound sound_id,
 	       entry->event_ca_id,
 	       entry->event_ca_description);
 
-	if (ca_proplist_create(&p) < 0)
+	if (ca_proplist_create (&p) < 0)
 		goto failed;
 
 	if (ca_proplist_sets (p, CA_PROP_EVENT_ID, entry->event_ca_id) < 0)
