@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2007-2008 Collabora Ltd.
  * Copyright (C) 2007 Marco Barisione <marco@barisione.org>
@@ -36,9 +35,10 @@
 #include <gio/gunixoutputstream.h>
 
 #include <telepathy-glib/proxy-subclass.h>
+#include <telepathy-glib/util.h>
 
 #include "empathy-tp-file.h"
-#include "empathy-contact-factory.h"
+#include "empathy-tp-contact-factory.h"
 #include "empathy-marshal.h"
 #include "empathy-time.h"
 #include "empathy-utils.h"
@@ -50,11 +50,18 @@
 
 /**
  * SECTION:empathy-tp-file
- * @short_description: File channel
- * @see_also: #EmpathyTpFile, #EmpathyContact, empathy_dispatcher_send_file()
- * @include: libempthy/empathy-tp-file.h
+ * @title: EmpathyTpFile
+ * @short_description: Object which represents a Telepathy file channel
+ * @include: libempathy/empathy-tp-file.h
  *
- * The #EmpathyTpFile object represents a Telepathy file channel.
+ * #EmpathyTpFile is an object which represents a Telepathy file channel.
+ */
+
+/**
+ * EmpathyTpFile:
+ * @parent: parent object
+ *
+ * Object which represents a Telepathy file channel.
  */
 
 /**
@@ -68,6 +75,7 @@
 
 #define N_BUFFERS 2
 #define BUFFER_SIZE 4096
+#define STALLED_TIMEOUT 5
 
 typedef struct {
   GInputStream *in;
@@ -276,9 +284,10 @@ copy_stream (GInputStream *in,
 /* EmpathyTpFile object */
 
 struct _EmpathyTpFilePriv {
-  EmpathyContactFactory *factory;
+  EmpathyTpContactFactory *factory;
   MissionControl *mc;
   TpChannel *channel;
+  gboolean ready;
 
   EmpathyContact *contact;
   GInputStream *in_stream;
@@ -296,8 +305,12 @@ struct _EmpathyTpFilePriv {
 
   gboolean incoming;
   TpFileTransferStateChangeReason state_change_reason;
-  time_t start_time;
-  gchar *unix_socket_path;
+  time_t last_update_time;
+  guint64 last_update_transferred_bytes;
+  gdouble speed;
+  gint remaining_time;
+  guint stalled_id;
+  GValue *socket_address;
   GCancellable *cancellable;
 };
 
@@ -306,6 +319,7 @@ enum {
   PROP_CHANNEL,
   PROP_STATE,
   PROP_INCOMING,
+  PROP_READY,
   PROP_FILENAME,
   PROP_SIZE,
   PROP_CONTENT_TYPE,
@@ -314,6 +328,13 @@ enum {
   PROP_CONTENT_HASH,
 };
 
+enum {
+	REFRESH,
+	LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL];
+
 G_DEFINE_TYPE (EmpathyTpFile, empathy_tp_file, G_TYPE_OBJECT);
 
 static void
@@ -370,7 +391,8 @@ tp_file_finalize (GObject *object)
     }
 
   g_free (tp_file->priv->filename);
-  g_free (tp_file->priv->unix_socket_path);
+  if (tp_file->priv->socket_address != NULL)
+    tp_g_value_slice_free (tp_file->priv->socket_address);
   g_free (tp_file->priv->description);
   g_free (tp_file->priv->content_hash);
   g_free (tp_file->priv->content_type);
@@ -387,15 +409,31 @@ tp_file_finalize (GObject *object)
   if (tp_file->priv->cancellable)
     g_object_unref (tp_file->priv->cancellable);
 
+  if (tp_file->priv->stalled_id != 0)
+    g_source_remove (tp_file->priv->stalled_id);
+
   G_OBJECT_CLASS (empathy_tp_file_parent_class)->finalize (object);
 }
 
+static gboolean
+tp_file_stalled_cb (EmpathyTpFile *tp_file)
+{
+  /* We didn't get transferred bytes update for a while, the transfer is
+   * stalled. */
+
+  tp_file->priv->speed = 0;
+  tp_file->priv->remaining_time = -1;
+  g_signal_emit (tp_file, signals[REFRESH], 0);
+
+  return FALSE;
+}
+
 static void
 tp_file_start_transfer (EmpathyTpFile *tp_file)
 {
   gint fd;
-  size_t path_len;
   struct sockaddr_un addr;
+  GArray *array;
 
   fd = socket (PF_UNIX, SOCK_STREAM, 0);
   if (fd < 0)
@@ -405,10 +443,11 @@ tp_file_start_transfer (EmpathyTpFile *tp_file)
       return;
     }
 
+  array = g_value_get_boxed (tp_file->priv->socket_address);
+
   memset (&addr, 0, sizeof (addr));
   addr.sun_family = AF_UNIX;
-  path_len = strlen (tp_file->priv->unix_socket_path);
-  strncpy (addr.sun_path, tp_file->priv->unix_socket_path, path_len);
+  strncpy (addr.sun_path, array->data, array->len);
 
   if (connect (fd, (struct sockaddr*) &addr, sizeof (addr)) < 0)
     {
@@ -420,7 +459,11 @@ tp_file_start_transfer (EmpathyTpFile *tp_file)
 
   DEBUG ("Start the transfer");
 
-  tp_file->priv->start_time = empathy_time_get_current ();
+  tp_file->priv->last_update_time = empathy_time_get_current ();
+  tp_file->priv->last_update_transferred_bytes = tp_file->priv->transferred_bytes;
+  tp_file->priv->stalled_id = g_timeout_add_seconds (STALLED_TIMEOUT,
+    (GSourceFunc) tp_file_stalled_cb, tp_file);
+
   tp_file->priv->cancellable = g_cancellable_new ();
   if (tp_file->priv->incoming)
     {
@@ -466,7 +509,7 @@ tp_file_state_changed_cb (TpChannel *channel,
    * transfer. The socket path could be NULL if we are not doing the actual
    * data transfer but are just an observer for the channel. */
   if (state == TP_FILE_TRANSFER_STATE_OPEN &&
-      tp_file->priv->unix_socket_path != NULL)
+      tp_file->priv->socket_address != NULL)
     tp_file_start_transfer (tp_file);
 
   tp_file->priv->state = state;
@@ -482,12 +525,145 @@ tp_file_transferred_bytes_changed_cb (TpChannel *channel,
                                       GObject *weak_object)
 {
   EmpathyTpFile *tp_file = EMPATHY_TP_FILE (weak_object);
+  time_t curr_time, elapsed_time;
+  guint64 transferred_bytes;
 
+  /* If we didn't progress since last update, return */
   if (tp_file->priv->transferred_bytes == count)
     return;
 
+  /* Update the transferred bytes count */
   tp_file->priv->transferred_bytes = count;
   g_object_notify (G_OBJECT (tp_file), "transferred-bytes");
+
+  /* We got a progress, reset the stalled timeout */
+  if (tp_file->priv->stalled_id != 0)
+    g_source_remove (tp_file->priv->stalled_id);
+  tp_file->priv->stalled_id = g_timeout_add_seconds (STALLED_TIMEOUT,
+    (GSourceFunc) tp_file_stalled_cb, tp_file);
+
+  /* Calculate the transfer speed and remaining time estimation. We recalculate
+   * that each second to get more dynamic values that react faster to network
+   * changes. This is better than calculating the average from the begining of
+   * the transfer, I think. */
+  curr_time = empathy_time_get_current ();
+  elapsed_time = curr_time - tp_file->priv->last_update_time;
+  if (elapsed_time >= 1)
+    {
+      transferred_bytes = count - tp_file->priv->last_update_transferred_bytes;
+      tp_file->priv->speed = (gdouble) transferred_bytes / (gdouble) elapsed_time;
+      tp_file->priv->remaining_time = (tp_file->priv->size - count) /
+        tp_file->priv->speed;
+      tp_file->priv->last_update_transferred_bytes = count;
+      tp_file->priv->last_update_time = curr_time;
+
+      g_signal_emit (tp_file, signals[REFRESH], 0);
+    }
+}
+
+static void
+tp_file_check_if_ready (EmpathyTpFile *tp_file)
+{
+  if (tp_file->priv->ready || tp_file->priv->contact == NULL ||
+      tp_file->priv->state == TP_FILE_TRANSFER_STATE_NONE)
+    return;
+
+  tp_file->priv->ready = TRUE;
+  g_object_notify (G_OBJECT (tp_file), "ready");
+}
+
+static void
+tp_file_got_contact_cb (EmpathyTpContactFactory *factory,
+                        EmpathyContact *contact,
+                        const GError *error,
+                        gpointer user_data,
+                        GObject *weak_object)
+{
+  EmpathyTpFile *tp_file = EMPATHY_TP_FILE (weak_object);
+
+  if (error)
+    {
+      DEBUG ("Error: %s", error->message);
+      empathy_tp_file_cancel (tp_file);
+      return;
+    }
+
+  tp_file->priv->contact = g_object_ref (contact);
+  tp_file_check_if_ready (tp_file);
+}
+
+static void
+tp_file_get_all_cb (TpProxy *proxy,
+                    GHashTable *properties,
+                    const GError *error,
+                    gpointer user_data,
+                    GObject *file_obj)
+{
+  EmpathyTpFile *tp_file = EMPATHY_TP_FILE (file_obj);
+
+  if (error)
+    {
+      DEBUG ("Error: %s", error->message);
+      tp_cli_channel_call_close (tp_file->priv->channel, -1, NULL, NULL, NULL,
+          NULL);
+      return;
+    }
+
+  tp_file->priv->size = g_value_get_uint64 (
+      g_hash_table_lookup (properties, "Size"));
+  g_object_notify (file_obj, "size");
+
+  tp_file->priv->state = g_value_get_uint (
+      g_hash_table_lookup (properties, "State"));
+  g_object_notify (file_obj, "state");
+
+  tp_file->priv->transferred_bytes = g_value_get_uint64 (
+      g_hash_table_lookup (properties, "TransferredBytes"));
+  g_object_notify (file_obj, "transferred-bytes");
+
+  tp_file->priv->filename = g_value_dup_string (
+      g_hash_table_lookup (properties, "Filename"));
+  g_object_notify (file_obj, "filename");
+
+  tp_file->priv->content_hash = g_value_dup_string (
+      g_hash_table_lookup (properties, "ContentHash"));
+  g_object_notify (file_obj, "content-hash");
+
+  tp_file->priv->content_hash_type = g_value_get_uint (
+      g_hash_table_lookup (properties, "ContentHashType"));
+  g_object_notify (file_obj, "content-hash-type");
+
+  tp_file->priv->content_type = g_value_dup_string (
+      g_hash_table_lookup (properties, "ContentType"));
+  g_object_notify (file_obj, "content-type");
+
+  tp_file->priv->description = g_value_dup_string (
+      g_hash_table_lookup (properties, "Description"));
+
+  tp_file_check_if_ready (tp_file);
+}
+
+static void
+tp_file_get_requested_cb (TpProxy *proxy,
+                          const GValue *requested,
+                          const GError *error,
+                          gpointer user_data,
+                          GObject *weak_object)
+{
+  EmpathyTpFile *tp_file = EMPATHY_TP_FILE (weak_object);
+
+  if (error)
+    {
+      DEBUG ("Error: %s", error->message);
+      tp_cli_channel_call_close (tp_file->priv->channel, -1, NULL, NULL, NULL,
+          NULL);
+      return;
+    }
+
+  tp_file->priv->incoming = !g_value_get_boolean (requested);
+  g_object_notify (G_OBJECT (tp_file), "incoming");
+
+  tp_file_check_if_ready (tp_file);
 }
 
 static GObject *
@@ -498,17 +674,18 @@ tp_file_constructor (GType type,
   GObject *file_obj;
   EmpathyTpFile *tp_file;
   TpHandle handle;
-  GHashTable *properties;
-  McAccount *account;
-  GValue *requested;
+  TpConnection *connection;
 
   file_obj = G_OBJECT_CLASS (empathy_tp_file_parent_class)->constructor (type,
       n_props, props);
 
   tp_file = EMPATHY_TP_FILE (file_obj);
 
-  tp_file->priv->factory = empathy_contact_factory_dup_singleton ();
+  connection = tp_channel_borrow_connection (tp_file->priv->channel);
+  tp_file->priv->factory = empathy_tp_contact_factory_dup_singleton (connection);
   tp_file->priv->mc = empathy_mission_control_dup_singleton ();
+  tp_file->priv->state_change_reason =
+      TP_FILE_TRANSFER_STATE_CHANGE_REASON_NONE;
 
   g_signal_connect (tp_file->priv->channel, "invalidated",
     G_CALLBACK (tp_file_invalidated_cb), tp_file);
@@ -521,50 +698,17 @@ tp_file_constructor (GType type,
       tp_file->priv->channel, tp_file_transferred_bytes_changed_cb,
       NULL, NULL, G_OBJECT (tp_file), NULL);
 
-  account = empathy_channel_get_account (tp_file->priv->channel);
+  tp_cli_dbus_properties_call_get (tp_file->priv->channel, -1,
+      TP_IFACE_CHANNEL, "Requested",
+      tp_file_get_requested_cb, NULL, NULL, file_obj);
+
+  tp_cli_dbus_properties_call_get_all (tp_file->priv->channel, -1,
+      TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER,
+      tp_file_get_all_cb, NULL, NULL, file_obj);
 
   handle = tp_channel_get_handle (tp_file->priv->channel, NULL);
-  tp_file->priv->contact = empathy_contact_factory_get_from_handle (
-      tp_file->priv->factory, account, (guint) handle);
-
-  tp_cli_dbus_properties_run_get_all (tp_file->priv->channel,
-      -1, TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER, &properties, NULL, NULL);
-
-  tp_cli_dbus_properties_run_get (tp_file->priv->channel,
-      -1, TP_IFACE_CHANNEL, "Requested", &requested, NULL, NULL);
-
-  tp_file->priv->size = g_value_get_uint64 (
-      g_hash_table_lookup (properties, "Size"));
-
-  tp_file->priv->state = g_value_get_uint (
-      g_hash_table_lookup (properties, "State"));
-
-  tp_file->priv->state_change_reason =
-      TP_FILE_TRANSFER_STATE_CHANGE_REASON_NONE;
-
-  tp_file->priv->transferred_bytes = g_value_get_uint64 (
-      g_hash_table_lookup (properties, "TransferredBytes"));
-
-  tp_file->priv->filename = g_value_dup_string (
-      g_hash_table_lookup (properties, "Filename"));
-
-  tp_file->priv->content_hash = g_value_dup_string (
-      g_hash_table_lookup (properties, "ContentHash"));
-
-  tp_file->priv->content_hash_type = g_value_get_uint (
-      g_hash_table_lookup (properties, "ContentHashType"));
-
-  tp_file->priv->content_type = g_value_dup_string (
-      g_hash_table_lookup (properties, "ContentType"));
-
-  tp_file->priv->description = g_value_dup_string (
-      g_hash_table_lookup (properties, "Description"));
-
-  tp_file->priv->incoming = !g_value_get_boolean (requested);
-
-  g_hash_table_destroy (properties);
-  g_object_unref (account);
-  g_value_unset (requested);
+  empathy_tp_contact_factory_get_from_handle (tp_file->priv->factory,
+      handle, tp_file_got_contact_cb, NULL, NULL, file_obj);
 
   return file_obj;
 }
@@ -608,6 +752,9 @@ tp_file_get_property (GObject *object,
       case PROP_TRANSFERRED_BYTES:
         g_value_set_uint64 (value, tp_file->priv->transferred_bytes);
         break;
+      case PROP_READY:
+        g_value_set_boolean (value, tp_file->priv->ready);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
         break;
@@ -683,12 +830,13 @@ tp_file_weak_notify_cb (gpointer channel,
 
 /**
  * empathy_tp_file_new:
- * @channel: a Telepathy channel
+ * @channel: a #TpChannel
  *
  * Creates a new #EmpathyTpFile wrapping @channel, or return a new ref to an
- * existing #EmpathyTpFile for that channel.
+ * existing #EmpathyTpFile for that channel. The returned #EmpathyTpFile
+ * should be unrefed with g_object_unref() when finished with.
  *
- * Returns: a new #EmpathyTpFile
+ * Return value: a new #EmpathyTpFile
  */
 EmpathyTpFile *
 empathy_tp_file_new (TpChannel *channel)
@@ -722,9 +870,9 @@ empathy_tp_file_new (TpChannel *channel)
  * empathy_tp_file_get_channel
  * @tp_file: an #EmpathyTpFile
  *
- * Returns the Telepathy file transfer channel
+ * Returns the #TpChannel associated with @tp_file.
  *
- * Returns: the #TpChannel
+ * Returns: the #TpChannel associated with @tp_file
  */
 TpChannel *
 empathy_tp_file_get_channel (EmpathyTpFile *tp_file)
@@ -742,6 +890,7 @@ tp_file_method_cb (TpChannel *channel,
                    GObject *weak_object)
 {
   EmpathyTpFile *tp_file = (EmpathyTpFile *) weak_object;
+  GArray *array;
 
   if (error)
     {
@@ -750,9 +899,35 @@ tp_file_method_cb (TpChannel *channel,
       return;
     }
 
-  tp_file->priv->unix_socket_path = g_value_dup_string (address);
+  if (G_VALUE_TYPE (address) == DBUS_TYPE_G_UCHAR_ARRAY)
+    {
+      tp_file->priv->socket_address = tp_g_value_slice_dup (address);
+    }
+  else if (G_VALUE_TYPE (address) == G_TYPE_STRING)
+    {
+      /* Old bugged version of telepathy-salut used to store the address
+       * as a 's' instead of an 'ay' */
+      const gchar *path;
 
-  DEBUG ("Got unix socket path: %s", tp_file->priv->unix_socket_path);
+      path = g_value_get_string (address);
+      array = g_array_sized_new (TRUE, FALSE, sizeof (gchar), strlen (path));
+      g_array_insert_vals (array, 0, path, strlen (path));
+
+      tp_file->priv->socket_address = tp_g_value_slice_new (
+          DBUS_TYPE_G_UCHAR_ARRAY);
+      g_value_set_boxed (tp_file->priv->socket_address, array);
+
+      g_array_free (array, TRUE);
+    }
+  else
+    {
+      DEBUG ("Wrong address type: %s", G_VALUE_TYPE_NAME (address));
+      empathy_tp_file_cancel (tp_file);
+      return;
+    }
+
+  array = g_value_get_boxed (tp_file->priv->socket_address);
+  DEBUG ("Got unix socket path: %s", array->data);
 
   if (tp_file->priv->state == TP_FILE_TRANSFER_STATE_OPEN)
     tp_file_start_transfer (tp_file);
@@ -766,7 +941,7 @@ tp_file_method_cb (TpChannel *channel,
  * @error: a #GError set if there is an error when opening @gfile
  *
  * Accepts a file transfer that's in the "local pending" state (i.e.
- * TP_FILE_TRANSFER_STATE_LOCAL_PENDING).
+ * %TP_FILE_TRANSFER_STATE_LOCAL_PENDING).
  */
 void
 empathy_tp_file_accept (EmpathyTpFile *tp_file,
@@ -784,6 +959,7 @@ empathy_tp_file_accept (EmpathyTpFile *tp_file,
   if (error && *error)
     return;
 
+  g_free (tp_file->priv->filename);
   tp_file->priv->filename = g_file_get_basename (gfile);
   g_object_notify (G_OBJECT (tp_file), "filename");
 
@@ -804,7 +980,7 @@ empathy_tp_file_accept (EmpathyTpFile *tp_file,
  * @error: a #GError set if there is an error when opening @gfile
  *
  * Offers a file transfer that's in the "not offered" state (i.e.
- * TP_FILE_TRANSFER_STATE_NOT_OFFERED).
+ * %TP_FILE_TRANSFER_STATE_NOT_OFFERED).
  */
 void
 empathy_tp_file_offer (EmpathyTpFile *tp_file, GFile *gfile, GError **error)
@@ -825,6 +1001,14 @@ empathy_tp_file_offer (EmpathyTpFile *tp_file, GFile *gfile, GError **error)
       &nothing, tp_file_method_cb, NULL, NULL, G_OBJECT (tp_file));
 }
 
+/**
+ * empathy_tp_file_get_contact:
+ * @tp_file: an #EmpathyTpFile
+ *
+ * Returns the #EmpathyContact that @tp_file is open with.
+ *
+ * Return value: the #EmpathyContact that @tp_file is open with.
+ */
 EmpathyContact *
 empathy_tp_file_get_contact (EmpathyTpFile *tp_file)
 {
@@ -839,6 +1023,14 @@ empathy_tp_file_get_filename (EmpathyTpFile *tp_file)
   return tp_file->priv->filename;
 }
 
+/**
+ * empathy_tp_file_is_incoming:
+ * @tp_file: an #EmpathyTpFile
+ *
+ * Returns whether @tp_file is incoming.
+ *
+ * Return value: %TRUE if the @tp_file is incoming, otherwise %FALSE
+ */
 gboolean
 empathy_tp_file_is_incoming (EmpathyTpFile *tp_file)
 {
@@ -846,6 +1038,16 @@ empathy_tp_file_is_incoming (EmpathyTpFile *tp_file)
   return tp_file->priv->incoming;
 }
 
+/**
+ * empathy_tp_file_get_state:
+ * @tp_file: an #EmpathyTpFile
+ * @reason: return location for state change reason, or %NULL
+ *
+ * Gets the current state of @tp_file. If @reason is not %NULL, then
+ * it is set to the reason of the last state change.
+ *
+ * Return value: a #TpFileTransferState
+ */
 TpFileTransferState
 empathy_tp_file_get_state (EmpathyTpFile *tp_file,
                            TpFileTransferStateChangeReason *reason)
@@ -859,6 +1061,14 @@ empathy_tp_file_get_state (EmpathyTpFile *tp_file,
   return tp_file->priv->state;
 }
 
+/**
+ * empathy_tp_file_get_size:
+ * @tp_file: an #EmpathyTpFile
+ *
+ * Gets the size of the file being transferred over @tp_file, in bytes.
+ *
+ * Return value: the size of the file being transferred, in bytes
+ */
 guint64
 empathy_tp_file_get_size (EmpathyTpFile *tp_file)
 {
@@ -867,6 +1077,14 @@ empathy_tp_file_get_size (EmpathyTpFile *tp_file)
   return tp_file->priv->size;
 }
 
+/**
+ * empathy_tp_file_get_transferred_bytes:
+ * @tp_file: an #EmpathyTpFile
+ *
+ * Gets the number of transferred bytes of @tp_file so far, in bytes.
+ *
+ * Return value: number of transferred bytes of @tp_file, in bytes
+ */
 guint64
 empathy_tp_file_get_transferred_bytes (EmpathyTpFile *tp_file)
 {
@@ -874,13 +1092,17 @@ empathy_tp_file_get_transferred_bytes (EmpathyTpFile *tp_file)
   return tp_file->priv->transferred_bytes;
 }
 
+/**
+ * empathy_tp_file_get_remaining_time:
+ * @tp_file: a #EmpathyTpFile
+ *
+ * Gets the estimated time remaining of @tp_file, in seconds.
+ *
+ * Return value: the estimated time remaining of @tp_file, in seconds
+ **/
 gint
 empathy_tp_file_get_remaining_time (EmpathyTpFile *tp_file)
 {
-  time_t curr_time, elapsed_time;
-  gdouble time_per_byte;
-  gdouble remaining_time;
-
   g_return_val_if_fail (EMPATHY_IS_TP_FILE (tp_file), -1);
 
   if (tp_file->priv->size == EMPATHY_TP_FILE_UNKNOWN_SIZE)
@@ -889,14 +1111,28 @@ empathy_tp_file_get_remaining_time (EmpathyTpFile *tp_file)
   if (tp_file->priv->transferred_bytes == tp_file->priv->size)
     return 0;
 
-  curr_time = empathy_time_get_current ();
-  elapsed_time = curr_time - tp_file->priv->start_time;
-  time_per_byte = (gdouble) elapsed_time /
-      (gdouble) tp_file->priv->transferred_bytes;
-  remaining_time = time_per_byte * (tp_file->priv->size -
-      tp_file->priv->transferred_bytes);
+  return tp_file->priv->remaining_time;
+}
 
-  return (gint) remaining_time;
+/**
+ * empathy_tp_file_get_speed:
+ * @tp_file: an #EmpathyTpFile
+ *
+ * Gets the current speed of the transfer @tp_file, in bytes per
+ * second.
+ *
+ * Return value: the current speed of the transfer @tp_file, in
+ *               bytes per second
+ **/
+gdouble
+empathy_tp_file_get_speed (EmpathyTpFile *tp_file)
+{
+  g_return_val_if_fail (EMPATHY_IS_TP_FILE (tp_file), 0);
+
+  if (tp_file->priv->transferred_bytes == tp_file->priv->size)
+    return 0;
+
+  return tp_file->priv->speed;
 }
 
 const gchar *
@@ -906,6 +1142,12 @@ empathy_tp_file_get_content_type (EmpathyTpFile *tp_file)
   return tp_file->priv->content_type;
 }
 
+/**
+ * empathy_tp_file_cancel:
+ * @tp_file: an #EmpathyTpFile
+ *
+ * Cancels the file transfer, @tp_file.
+ */
 void
 empathy_tp_file_cancel (EmpathyTpFile *tp_file)
 {
@@ -919,10 +1161,24 @@ empathy_tp_file_cancel (EmpathyTpFile *tp_file)
     g_cancellable_cancel (tp_file->priv->cancellable);
 }
 
-void
-empathy_tp_file_close (EmpathyTpFile *tp_file)
+/**
+ * empathy_tp_file_is_ready:
+ * @tp_file: an #EmpathyTpFile
+ *
+ * Returns whether the file channel @tp_file is ready for use.
+ *
+ * @tp_file is classed as ready if its state is no longer
+ * %TP_FILE_TRANSFER_STATE_NONE, or if details about the remote
+ * contact have been fully received.
+ *
+ * Return value: %TRUE if @tp_file is ready for use
+ */
+gboolean
+empathy_tp_file_is_ready (EmpathyTpFile *tp_file)
 {
-  empathy_tp_file_cancel (tp_file);
+  g_return_val_if_fail (EMPATHY_IS_TP_FILE (tp_file), FALSE);
+
+  return tp_file->priv->ready;
 }
 
 static void
@@ -936,6 +1192,12 @@ empathy_tp_file_class_init (EmpathyTpFileClass *klass)
   object_class->set_property = tp_file_set_property;
 
   /* Construct-only properties */
+
+  /**
+   * EmpathyTpFile:channel:
+   *
+   * The #TpChannel associated with the #EmpathyTpFile.
+   */
   g_object_class_install_property (object_class,
       PROP_CHANNEL,
       g_param_spec_object ("channel",
@@ -945,6 +1207,11 @@ empathy_tp_file_class_init (EmpathyTpFileClass *klass)
           G_PARAM_READWRITE |
           G_PARAM_CONSTRUCT_ONLY));
 
+  /**
+   * EmpathyTpFile:state:
+   *
+   * The #TpFileTransferState of the #EmpathyTpFile.
+   */
   g_object_class_install_property (object_class,
       PROP_STATE,
       g_param_spec_uint ("state",
@@ -956,6 +1223,11 @@ empathy_tp_file_class_init (EmpathyTpFileClass *klass)
           G_PARAM_READWRITE |
           G_PARAM_CONSTRUCT));
 
+  /**
+   * EmpathyTpFile:incoming:
+   *
+   * Whether the #EmpathyTpFile is incoming.
+   */
   g_object_class_install_property (object_class,
       PROP_INCOMING,
       g_param_spec_boolean ("incoming",
@@ -965,6 +1237,25 @@ empathy_tp_file_class_init (EmpathyTpFileClass *klass)
           G_PARAM_READWRITE |
           G_PARAM_CONSTRUCT));
 
+  /**
+   * EmpathyTpFile:ready:
+   *
+   * Whether the #EmpathyTpFile is ready to use. This property returns
+   * the same as empathy_tp_file_is_ready().
+   */
+  g_object_class_install_property (object_class,
+      PROP_READY,
+      g_param_spec_boolean ("ready",
+          "ready",
+          "Whether the object is ready",
+          FALSE,
+          G_PARAM_READABLE));
+
+  /**
+   * EmpathyTpFile:filename:
+   *
+   * The name of the file being transferred.
+   */
   g_object_class_install_property (object_class,
       PROP_FILENAME,
       g_param_spec_string ("filename",
@@ -973,6 +1264,11 @@ empathy_tp_file_class_init (EmpathyTpFileClass *klass)
           "",
           G_PARAM_READWRITE));
 
+  /**
+   * EmpathyTpFile:size:
+   *
+   * The size of the file being transferred.
+   */
   g_object_class_install_property (object_class,
       PROP_SIZE,
       g_param_spec_uint64 ("size",
@@ -983,6 +1279,11 @@ empathy_tp_file_class_init (EmpathyTpFileClass *klass)
           G_MAXUINT64,
           G_PARAM_READWRITE));
 
+  /**
+   * EmpathyTpFile:content-type:
+   *
+   * The content-type of the file being transferred.
+   */
   g_object_class_install_property (object_class,
       PROP_CONTENT_TYPE,
       g_param_spec_string ("content-type",
@@ -991,6 +1292,12 @@ empathy_tp_file_class_init (EmpathyTpFileClass *klass)
           "",
           G_PARAM_READWRITE));
 
+  /**
+   * EmpathyTpFile:content-hash-type:
+   *
+   * The type of hash type stored in #EmpathyTpFile:content-hash,
+   * from #TpFileHashType.
+   */
   g_object_class_install_property (object_class,
       PROP_CONTENT_HASH_TYPE,
       g_param_spec_uint ("content-hash-type",
@@ -1001,6 +1308,11 @@ empathy_tp_file_class_init (EmpathyTpFileClass *klass)
           0,
           G_PARAM_READWRITE));
 
+  /**
+   * EmpathyTpFile:content-hash:
+   *
+   * A hash of the contents of the file being transferred.
+   */
   g_object_class_install_property (object_class,
       PROP_CONTENT_HASH,
       g_param_spec_string ("content-hash",
@@ -1009,6 +1321,11 @@ empathy_tp_file_class_init (EmpathyTpFileClass *klass)
           "",
           G_PARAM_READWRITE));
 
+  /**
+   * EmpathyTpFile:transferred-bytes:
+   *
+   * The number of bytes transferred in the #EmpathyTpFile.
+   */
   g_object_class_install_property (object_class,
       PROP_TRANSFERRED_BYTES,
       g_param_spec_uint64 ("transferred-bytes",
@@ -1019,6 +1336,23 @@ empathy_tp_file_class_init (EmpathyTpFileClass *klass)
           0,
           G_PARAM_READWRITE));
 
+  /**
+   * EmpathyTpFile::refresh:
+   * @tp_file: the #EmpathyTpFile
+   *
+   * The progress of @tp_file has changed. This can either be an update
+   * in the number of bytes transferred, or it can be to inform of the
+   * transfer stalling.
+   *
+   * This signal is designed for clients to provide more user feedback
+   * when something to do with @tp_file changes. To avoid emitting this
+   * signal too much, it is guaranteed that it will only ever be fired
+   * at least every two seconds.
+   */
+  signals[REFRESH] = g_signal_new ("refresh", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+      g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+
   g_type_class_add_private (object_class, sizeof (EmpathyTpFilePriv));
 }
 
