@@ -15,7 +15,7 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- * 
+ *
  * Authors: Xavier Claessens <xclaesse@gmail.com>
  */
 
@@ -42,11 +42,13 @@ typedef struct {
 	TpChannel    *channel;
 	McAccount    *account;
 	gboolean      is_listing;
+	gboolean      start_requested;
 } EmpathyTpRoomlistPriv;
 
 enum {
 	NEW_ROOM,
 	DESTROY,
+	ERROR,
 	LAST_SIGNAL
 };
 
@@ -122,6 +124,10 @@ tp_roomlist_got_rooms_cb (TpChannel       *channel,
 	for (i = 0; i < rooms->len; i++) {
 		const GValue *room_name_value;
 		const GValue *handle_name_value;
+		const GValue *room_members_value;
+		const GValue *room_subject_value;
+		const GValue *room_invite_value;
+		const GValue *room_password_value;
 		GValueArray  *room_struct;
 		guint         handle;
 		const gchar  *channel_type;
@@ -134,6 +140,10 @@ tp_roomlist_got_rooms_cb (TpChannel       *channel,
 		info = g_value_get_boxed (g_value_array_get_nth (room_struct, 2));
 		room_name_value = g_hash_table_lookup (info, "name");
 		handle_name_value = g_hash_table_lookup (info, "handle-name");
+		room_subject_value = g_hash_table_lookup (info, "subject");
+		room_members_value = g_hash_table_lookup (info, "members");
+		room_invite_value = g_hash_table_lookup (info, "invite-only");
+		room_password_value = g_hash_table_lookup (info, "password");
 
 		if (tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TEXT)) {
 			continue;
@@ -146,6 +156,26 @@ tp_roomlist_got_rooms_cb (TpChannel       *channel,
 						   g_value_get_string (room_name_value));
 		}
 
+		if (room_members_value != NULL) {
+			empathy_chatroom_set_members_count (chatroom,
+						   g_value_get_uint (room_members_value));
+		}
+
+		if (room_subject_value != NULL) {
+			empathy_chatroom_set_subject (chatroom,
+						   g_value_get_string (room_subject_value));
+		}
+
+		if (room_invite_value != NULL) {
+			empathy_chatroom_set_invite_only (chatroom,
+						   g_value_get_boolean (room_invite_value));
+		}
+
+		if (room_password_value != NULL) {
+			empathy_chatroom_set_need_password (chatroom,
+						   g_value_get_boolean (room_password_value));
+		}
+
 		if (handle_name_value != NULL) {
 			empathy_chatroom_set_room (chatroom,
 						   g_value_get_string (handle_name_value));
@@ -208,25 +238,43 @@ tp_roomlist_invalidated_cb (TpChannel         *channel,
 }
 
 static void
-tp_roomlist_request_channel_cb (TpConnection *connection,
-				const gchar  *object_path,
-				const GError *error,
-				gpointer      user_data,
-				GObject      *list)
+call_list_rooms_cb (TpChannel *proxy,
+		    const GError *error,
+		    gpointer list,
+		    GObject *weak_object)
 {
+	if (error != NULL) {
+		DEBUG ("Error listing rooms: %s", error->message);
+		g_signal_emit_by_name (list, "error::start", error);
+	}
+}
+
+static void
+stop_listing_cb (TpChannel *proxy,
+		 const GError *error,
+		 gpointer list,
+		 GObject *weak_object)
+{
+	if (error != NULL) {
+		DEBUG ("Error on stop listing: %s", error->message);
+		g_signal_emit_by_name (list, "error::stop", error);
+	}
+}
+
+static void
+channel_ready_cb (TpChannel *channel,
+		  const GError *error,
+		  gpointer user_data)
+{
+	EmpathyTpRoomlist *list = EMPATHY_TP_ROOMLIST (user_data);
 	EmpathyTpRoomlistPriv *priv = GET_PRIV (list);
 
-	if (error) {
-		DEBUG ("Error requesting channel: %s", error->message);
+	if (error != NULL) {
+		DEBUG ("Channel invalidated: %s", error->message);
+		g_signal_emit (list, signals[DESTROY], 0);
 		return;
 	}
 
-	priv->channel = tp_channel_new (priv->connection, object_path,
-					TP_IFACE_CHANNEL_TYPE_ROOM_LIST,
-					TP_HANDLE_TYPE_NONE,
-					0, NULL);
-	tp_channel_run_until_ready (priv->channel, NULL, NULL);
-
 	g_signal_connect (priv->channel, "invalidated",
 			  G_CALLBACK (tp_roomlist_invalidated_cb),
 			  list);
@@ -246,6 +294,33 @@ tp_roomlist_request_channel_cb (TpConnection *connection,
 							      tp_roomlist_get_listing_rooms_cb,
 							      NULL, NULL,
 							      G_OBJECT (list));
+
+	if (priv->start_requested == TRUE) {
+		tp_cli_channel_type_room_list_call_list_rooms (priv->channel, -1,
+			call_list_rooms_cb, list, NULL, NULL);
+		priv->start_requested = FALSE;
+	}
+}
+
+static void
+tp_roomlist_request_channel_cb (TpConnection *connection,
+				const gchar  *object_path,
+				const GError *error,
+				gpointer      user_data,
+				GObject      *list)
+{
+	EmpathyTpRoomlistPriv *priv = GET_PRIV (list);
+
+	if (error) {
+		DEBUG ("Error requesting channel: %s", error->message);
+		return;
+	}
+
+	priv->channel = tp_channel_new (priv->connection, object_path,
+					TP_IFACE_CHANNEL_TYPE_ROOM_LIST,
+					TP_HANDLE_TYPE_NONE,
+					0, NULL);
+	tp_channel_call_when_ready (priv->channel, channel_ready_cb, list);
 }
 
 static void
@@ -380,6 +455,16 @@ empathy_tp_roomlist_class_init (EmpathyTpRoomlistClass *klass)
 			      G_TYPE_NONE,
 			      0);
 
+	signals[ERROR] =
+		g_signal_new ("error",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+			      0,
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__POINTER,
+			      G_TYPE_NONE,
+			      1, G_TYPE_POINTER);
+
 	g_type_class_add_private (object_class, sizeof (EmpathyTpRoomlistPriv));
 }
 
@@ -390,6 +475,8 @@ empathy_tp_roomlist_init (EmpathyTpRoomlist *list)
 		EMPATHY_TYPE_TP_ROOMLIST, EmpathyTpRoomlistPriv);
 
 	list->priv = priv;
+	priv->start_requested = FALSE;
+	priv->is_listing = FALSE;
 }
 
 EmpathyTpRoomlist *
@@ -430,10 +517,12 @@ empathy_tp_roomlist_start (EmpathyTpRoomlist *list)
 	EmpathyTpRoomlistPriv *priv = GET_PRIV (list);
 
 	g_return_if_fail (EMPATHY_IS_TP_ROOMLIST (list));
-	g_return_if_fail (TP_IS_CHANNEL (priv->channel));
-
-	tp_cli_channel_type_room_list_call_list_rooms (priv->channel, -1,
-						       NULL, NULL, NULL, NULL);
+	if (priv->channel != NULL) {
+		tp_cli_channel_type_room_list_call_list_rooms (priv->channel, -1,
+			call_list_rooms_cb, list, NULL, NULL);
+	} else {
+		priv->start_requested = TRUE;
+	}
 }
 
 void
@@ -445,6 +534,6 @@ empathy_tp_roomlist_stop (EmpathyTpRoomlist *list)
 	g_return_if_fail (TP_IS_CHANNEL (priv->channel));
 
 	tp_cli_channel_type_room_list_call_stop_listing (priv->channel, -1,
-							 NULL, NULL, NULL, NULL);
+							 stop_listing_cb, list, NULL, NULL);
 }
 
