@@ -15,9 +15,9 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- * 
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ *
  * Authors: Mikael Hallendal <micke@imendio.com>
  *          Richard Hult <richard@imendio.com>
  *          Martyn Russell <martyn@imendio.com>
@@ -31,18 +31,17 @@
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade.h>
 #include <glib/gi18n.h>
 #include <libnotify/notification.h>
 
 #include <telepathy-glib/util.h>
 #include <libmissioncontrol/mission-control.h>
 
-#include <libempathy/empathy-contact-factory.h>
 #include <libempathy/empathy-contact.h>
 #include <libempathy/empathy-message.h>
 #include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-chatroom-manager.h>
+#include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-utils.h>
 
 #include <libempathy-gtk/empathy-images.h>
@@ -75,24 +74,19 @@ typedef struct {
 	NotifyNotification *notification;
 
 	/* Menu items. */
-	GtkWidget   *menu_conv_clear;
-	GtkWidget   *menu_conv_insert_smiley;
-	GtkWidget   *menu_conv_contact;
-	GtkWidget   *menu_conv_favorite;
-	GtkWidget   *menu_conv_close;
+	GtkUIManager *ui_manager;
+	GtkAction   *menu_conv_insert_smiley;
+	GtkAction   *menu_conv_favorite;
 
-	GtkWidget   *menu_edit_cut;
-	GtkWidget   *menu_edit_copy;
-	GtkWidget   *menu_edit_paste;
+	GtkAction   *menu_edit_cut;
+	GtkAction   *menu_edit_copy;
+	GtkAction   *menu_edit_paste;
 
-	GtkWidget   *menu_tabs_next;
-	GtkWidget   *menu_tabs_prev;
-	GtkWidget   *menu_tabs_left;
-	GtkWidget   *menu_tabs_right;
-	GtkWidget   *menu_tabs_detach;
-	
-	GtkWidget   *menu_help_contents;
-	GtkWidget   *menu_help_about;
+	GtkAction   *menu_tabs_next;
+	GtkAction   *menu_tabs_prev;
+	GtkAction   *menu_tabs_left;
+	GtkAction   *menu_tabs_right;
+	GtkAction   *menu_tabs_detach;
 } EmpathyChatWindowPriv;
 
 static GList *chat_windows = NULL;
@@ -157,7 +151,7 @@ chat_window_find_chat (EmpathyChat *chat)
 }
 
 static void
-chat_window_close_clicked_cb (GtkWidget  *button,
+chat_window_close_clicked_cb (GtkAction   *action,
 			      EmpathyChat *chat)
 {
 	EmpathyChatWindow *window;
@@ -284,6 +278,8 @@ chat_window_update (EmpathyChatWindow *window)
 	gboolean               avatar_in_icon;
 	GtkWidget             *chat;
 	GtkWidget             *chat_close_button;
+	GtkWidget             *submenu;
+	GtkWidget             *menu;
 
 	/* Get information */
 	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
@@ -297,12 +293,19 @@ chat_window_update (EmpathyChatWindow *window)
 	DEBUG ("Update window");
 
 	/* Update menu */
-	gtk_widget_set_sensitive (priv->menu_tabs_next, !last_page);
-	gtk_widget_set_sensitive (priv->menu_tabs_prev, !first_page);
-	gtk_widget_set_sensitive (priv->menu_tabs_detach, num_pages > 1);
-	gtk_widget_set_sensitive (priv->menu_tabs_left, !first_page);
-	gtk_widget_set_sensitive (priv->menu_tabs_right, !last_page);
-	gtk_widget_set_sensitive (priv->menu_conv_insert_smiley, is_connected);
+	gtk_action_set_sensitive (priv->menu_tabs_next, !last_page);
+	gtk_action_set_sensitive (priv->menu_tabs_prev, !first_page);
+	gtk_action_set_sensitive (priv->menu_tabs_detach, num_pages > 1);
+	gtk_action_set_sensitive (priv->menu_tabs_left, !first_page);
+	gtk_action_set_sensitive (priv->menu_tabs_right, !last_page);
+	gtk_action_set_sensitive (priv->menu_conv_insert_smiley, is_connected);
+
+	/* Update Contact menu */
+	menu = gtk_ui_manager_get_widget (priv->ui_manager,
+		"/chats_menubar/menu_contact");
+	submenu = empathy_chat_get_contact_menu (priv->current_chat);
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);
+	gtk_widget_show (menu);
 
 	/* Update window title */
 	if (n_chats == 1) {
@@ -489,25 +492,15 @@ chat_window_insert_smiley_activate_cb (EmpathySmileyManager *manager,
 }
 
 static void
-chat_window_conv_activate_cb (GtkWidget         *menuitem,
+chat_window_conv_activate_cb (GtkAction         *action,
 			      EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
-	GtkWidget             *submenu = NULL;
-
-	/* Contact submenu */
-	submenu = empathy_chat_get_contact_menu (priv->current_chat);
-	if (submenu) {
-		gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->menu_conv_contact),
-					   submenu);
-		gtk_widget_show (priv->menu_conv_contact);
-		gtk_widget_show (submenu);
-	} else {
-		gtk_widget_hide (priv->menu_conv_contact);
-	}
+	gboolean               is_room;
 
 	/* Favorite room menu */
-	if (empathy_chat_is_room (priv->current_chat)) {
+	is_room = empathy_chat_is_room (priv->current_chat);
+	if (is_room) {
 		const gchar *room;
 		McAccount   *account;
 		gboolean     found;
@@ -518,16 +511,14 @@ chat_window_conv_activate_cb (GtkWidget         *menuitem,
 						       account, room) != NULL;
 
 		DEBUG ("This room %s favorite", found ? "is" : "is not");
-		gtk_check_menu_item_set_active (
-			GTK_CHECK_MENU_ITEM (priv->menu_conv_favorite), found);
-		gtk_widget_show (priv->menu_conv_favorite);
-	} else {
-		gtk_widget_hide (priv->menu_conv_favorite);
+		gtk_toggle_action_set_active (
+			GTK_TOGGLE_ACTION (priv->menu_conv_favorite), found);
 	}
+	gtk_action_set_visible (priv->menu_conv_favorite, is_room);
 }
 
 static void
-chat_window_clear_activate_cb (GtkWidget        *menuitem,
+chat_window_clear_activate_cb (GtkAction         *action,
 			       EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
@@ -536,7 +527,7 @@ chat_window_clear_activate_cb (GtkWidget        *menuitem,
 }
 
 static void
-chat_window_favorite_toggled_cb (GtkCheckMenuItem  *menuitem,
+chat_window_favorite_toggled_cb (GtkToggleAction   *toggle_action,
 				 EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
@@ -545,7 +536,7 @@ chat_window_favorite_toggled_cb (GtkCheckMenuItem  *menuitem,
 	const gchar           *room;
 	EmpathyChatroom       *chatroom;
 
-	active = gtk_check_menu_item_get_active (menuitem);
+	active = gtk_toggle_action_get_active (toggle_action);
 	account = empathy_chat_get_account (priv->current_chat);
 	room = empathy_chat_get_id (priv->current_chat);
 
@@ -625,7 +616,7 @@ chat_window_configure_event_cb (GtkWidget         *widget,
 }
 
 static void
-chat_window_close_activate_cb (GtkWidget        *menuitem,
+chat_window_close_activate_cb (GtkAction         *action,
 			       EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv;
@@ -638,7 +629,7 @@ chat_window_close_activate_cb (GtkWidget        *menuitem,
 }
 
 static void
-chat_window_edit_activate_cb (GtkWidget        *menuitem,
+chat_window_edit_activate_cb (GtkAction         *action,
 			      EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv;
@@ -651,32 +642,32 @@ chat_window_edit_activate_cb (GtkWidget        *menuitem,
 	g_return_if_fail (priv->current_chat != NULL);
 
 	if (!empathy_chat_get_tp_chat (priv->current_chat)) {
-		gtk_widget_set_sensitive (priv->menu_edit_copy, FALSE);
-		gtk_widget_set_sensitive (priv->menu_edit_cut, FALSE);
-		gtk_widget_set_sensitive (priv->menu_edit_paste, FALSE);
+		gtk_action_set_sensitive (priv->menu_edit_copy, FALSE);
+		gtk_action_set_sensitive (priv->menu_edit_cut, FALSE);
+		gtk_action_set_sensitive (priv->menu_edit_paste, FALSE);
 		return;
 	}
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (priv->current_chat->input_text_view));
 	if (gtk_text_buffer_get_has_selection (buffer)) {
-		gtk_widget_set_sensitive (priv->menu_edit_copy, TRUE);
-		gtk_widget_set_sensitive (priv->menu_edit_cut, TRUE);
+		gtk_action_set_sensitive (priv->menu_edit_copy, TRUE);
+		gtk_action_set_sensitive (priv->menu_edit_cut, TRUE);
 	} else {
 		gboolean selection;
 
 		selection = empathy_chat_view_get_has_selection (priv->current_chat->view);
 
-		gtk_widget_set_sensitive (priv->menu_edit_cut, FALSE);
-		gtk_widget_set_sensitive (priv->menu_edit_copy, selection);
+		gtk_action_set_sensitive (priv->menu_edit_cut, FALSE);
+		gtk_action_set_sensitive (priv->menu_edit_copy, selection);
 	}
 
 	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
 	text_available = gtk_clipboard_wait_is_text_available (clipboard);
-	gtk_widget_set_sensitive (priv->menu_edit_paste, text_available);
+	gtk_action_set_sensitive (priv->menu_edit_paste, text_available);
 }
 
 static void
-chat_window_cut_activate_cb (GtkWidget        *menuitem,
+chat_window_cut_activate_cb (GtkAction         *action,
 			     EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv;
@@ -689,7 +680,7 @@ chat_window_cut_activate_cb (GtkWidget        *menuitem,
 }
 
 static void
-chat_window_copy_activate_cb (GtkWidget        *menuitem,
+chat_window_copy_activate_cb (GtkAction         *action,
 			      EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv;
@@ -702,7 +693,7 @@ chat_window_copy_activate_cb (GtkWidget        *menuitem,
 }
 
 static void
-chat_window_paste_activate_cb (GtkWidget        *menuitem,
+chat_window_paste_activate_cb (GtkAction         *action,
 			       EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv;
@@ -715,7 +706,7 @@ chat_window_paste_activate_cb (GtkWidget        *menuitem,
 }
 
 static void
-chat_window_tabs_left_activate_cb (GtkWidget        *menuitem,
+chat_window_tabs_left_activate_cb (GtkAction         *action,
 				   EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv;
@@ -736,7 +727,7 @@ chat_window_tabs_left_activate_cb (GtkWidget        *menuitem,
 }
 
 static void
-chat_window_tabs_right_activate_cb (GtkWidget        *menuitem,
+chat_window_tabs_right_activate_cb (GtkAction         *action,
 				    EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv;
@@ -754,7 +745,7 @@ chat_window_tabs_right_activate_cb (GtkWidget        *menuitem,
 }
 
 static void
-chat_window_detach_activate_cb (GtkWidget        *menuitem,
+chat_window_detach_activate_cb (GtkAction         *action,
 				EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv;
@@ -773,15 +764,17 @@ chat_window_detach_activate_cb (GtkWidget        *menuitem,
 }
 
 static void
-chat_window_help_contents_cb (GtkWidget         *menuitem,
-			      EmpathyChatWindow *window)
+chat_window_help_contents_activate_cb (GtkAction         *action,
+				       EmpathyChatWindow *window)
 {
-	empathy_url_show (menuitem, "ghelp:empathy?chat");
+	EmpathyChatWindowPriv *priv = GET_PRIV (window);
+
+	empathy_url_show (priv->dialog, "ghelp:empathy?chat");
 }
 
 static void
-chat_window_help_about_cb (GtkWidget         *menuitem,
-			   EmpathyChatWindow *window)
+chat_window_help_about_activate_cb (GtkAction         *action,
+				    EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
 
@@ -956,9 +949,9 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 	if (has_focus && priv->current_chat == chat) {
 		return;
 	}
-	
-	/* If empathy_chat_is_room() returns TRUE, that means it's a named MUC.
-	 * If empathy_chat_get_remote_contact() returns NULL, that means it's
+
+	/* If empathy_chat_is_room () returns TRUE, that means it's a named MUC.
+	 * If empathy_chat_get_remote_contact () returns NULL, that means it's
 	 * an unamed MUC (msn-like).
 	 * In case of a MUC, we set urgency only if the message contains our
 	 * alias. */
@@ -1164,17 +1157,33 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		McAccount             *account;
 		const gchar           *id;
 		gchar                **strv;
+		const gchar           *account_id;
+		const gchar           *contact_id;
 
 		id = (const gchar*) selection->data;
 
 		DEBUG ("DND contact from roster with id:'%s'", id);
 		
 		strv = g_strsplit (id, "/", 2);
-		account = mc_account_lookup (strv[0]);
-		chat = empathy_chat_window_find_chat (account, strv[1]);
+		account_id = strv[0];
+		contact_id = strv[1];
+		account = mc_account_lookup (account_id);
+		chat = empathy_chat_window_find_chat (account, contact_id);
 
 		if (!chat) {
-			empathy_dispatcher_chat_with_contact_id (account, strv[2], NULL, NULL);
+			EmpathyAccountManager *account_manager;
+			TpConnection *connection;
+
+			account_manager = empathy_account_manager_dup_singleton ();
+			connection = empathy_account_manager_get_connection (
+				account_manager, account);
+
+			if (connection) {
+				empathy_dispatcher_chat_with_contact_id (
+					connection, contact_id, NULL, NULL);
+			}
+
+			g_object_unref (account_manager);
 			g_object_unref (account);
 			g_strfreev (strv);
 			return;
@@ -1200,7 +1209,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		/* We should return TRUE to remove the data when doing
 		 * GDK_ACTION_MOVE, but we don't here otherwise it has
 		 * weird consequences, and we handle that internally
-		 * anyway with add_chat() and remove_chat().
+		 * anyway with add_chat () and remove_chat ().
 		 */
 		gtk_drag_finish (context, TRUE, FALSE, time);
 	}
@@ -1210,7 +1219,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 
 		DEBUG ("DND tab");
 
-		chat = (void*) selection->data;
+		chat = (void *) selection->data;
 		old_window = chat_window_find_chat (*chat);
 
 		if (old_window) {
@@ -1231,7 +1240,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		/* We should return TRUE to remove the data when doing
 		 * GDK_ACTION_MOVE, but we don't here otherwise it has
 		 * weird consequences, and we handle that internally
-		 * anyway with add_chat() and remove_chat().
+		 * anyway with add_chat () and remove_chat ().
 		 */
 		gtk_drag_finish (context, TRUE, FALSE, time);
 	} else {
@@ -1251,6 +1260,7 @@ chat_window_finalize (GObject *object)
 
 	DEBUG ("Finalized: %p", object);
 
+	g_object_unref (priv->ui_manager);
 	g_object_unref (priv->chatroom_manager);
 	if (priv->save_geometry_id != 0) {
 		g_source_remove (priv->save_geometry_id);
@@ -1293,11 +1303,11 @@ empathy_chat_window_class_init (EmpathyChatWindowClass *klass)
 static void
 empathy_chat_window_init (EmpathyChatWindow *window)
 {
-	GladeXML              *glade;
+	GtkBuilder            *gui;
 	GtkAccelGroup         *accel_group;
 	GClosure              *closure;
-	GtkWidget             *menu_conv;
 	GtkWidget             *menu;
+	GtkWidget             *submenu;
 	gint                   i;
 	GtkWidget             *chat_vbox;
 	gchar                 *filename;
@@ -1306,18 +1316,13 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 		EMPATHY_TYPE_CHAT_WINDOW, EmpathyChatWindowPriv);
 
 	window->priv = priv;
-	filename = empathy_file_lookup ("empathy-chat-window.glade", "src");
-	glade = empathy_glade_get_file (filename,
-				       "chat_window",
-				       NULL,
+	filename = empathy_file_lookup ("empathy-chat-window.ui", "src");
+	gui = empathy_builder_get_file (filename,
 				       "chat_window", &priv->dialog,
 				       "chat_vbox", &chat_vbox,
-				       "menu_conv", &menu_conv,
-				       "menu_conv_clear", &priv->menu_conv_clear,
+				       "ui_manager", &priv->ui_manager,
 				       "menu_conv_insert_smiley", &priv->menu_conv_insert_smiley,
-				       "menu_conv_contact", &priv->menu_conv_contact,
 				       "menu_conv_favorite", &priv->menu_conv_favorite,
-				       "menu_conv_close", &priv->menu_conv_close,
 				       "menu_edit_cut", &priv->menu_edit_cut,
 				       "menu_edit_copy", &priv->menu_edit_copy,
 				       "menu_edit_paste", &priv->menu_edit_paste,
@@ -1326,13 +1331,10 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 				       "menu_tabs_left", &priv->menu_tabs_left,
 				       "menu_tabs_right", &priv->menu_tabs_right,
 				       "menu_tabs_detach", &priv->menu_tabs_detach,
-				       "menu_help_contents", &priv->menu_help_contents,
-				       "menu_help_about", &priv->menu_help_about,
 				       NULL);
 	g_free (filename);
 
-	empathy_glade_connect (glade,
-			      window,
+	empathy_builder_connect (gui, window,
 			      "chat_window", "configure-event", chat_window_configure_event_cb,
 			      "menu_conv", "activate", chat_window_conv_activate_cb,
 			      "menu_conv_clear", "activate", chat_window_clear_activate_cb,
@@ -1345,16 +1347,17 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			      "menu_tabs_left", "activate", chat_window_tabs_left_activate_cb,
 			      "menu_tabs_right", "activate", chat_window_tabs_right_activate_cb,
 			      "menu_tabs_detach", "activate", chat_window_detach_activate_cb,
-			      "menu_help_contents", "activate", chat_window_help_contents_cb,
-			      "menu_help_about", "activate", chat_window_help_about_cb,
+			      "menu_help_contents", "activate", chat_window_help_contents_activate_cb,
+			      "menu_help_about", "activate", chat_window_help_about_activate_cb,
 			      NULL);
 
-	g_object_unref (glade);
+	g_object_ref (priv->ui_manager);
+	g_object_unref (gui);
 
 	priv->chatroom_manager = empathy_chatroom_manager_dup_singleton (NULL);
 
 	priv->notebook = gtk_notebook_new ();
- 	gtk_notebook_set_group (GTK_NOTEBOOK (priv->notebook), "EmpathyChatWindow"); 
+ 	gtk_notebook_set_group (GTK_NOTEBOOK (priv->notebook), "EmpathyChatWindow");
 	gtk_box_pack_start (GTK_BOX (chat_vbox), priv->notebook, TRUE, TRUE, 0);
 	gtk_widget_show (priv->notebook);
 
@@ -1377,14 +1380,15 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 
 	/* Set up smiley menu */
 	smiley_manager = empathy_smiley_manager_dup_singleton ();
-	menu = empathy_smiley_menu_new (smiley_manager,
-					chat_window_insert_smiley_activate_cb,
-					window);
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->menu_conv_insert_smiley),
-				   menu);
+	submenu = empathy_smiley_menu_new (smiley_manager,
+					   chat_window_insert_smiley_activate_cb,
+					   window);
+	menu = gtk_ui_manager_get_widget (priv->ui_manager,
+		"/chats_menubar/menu_conv/menu_conv_insert_smiley");
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);
 	g_object_unref (smiley_manager);
 
-	/* Set up signals we can't do with glade since we may need to
+	/* Set up signals we can't do with ui file since we may need to
 	 * block/unblock them at some later stage.
 	 */
 
@@ -1522,7 +1526,7 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 		}
 		
 		if (w > 0 && h > 0) {
-			/* Use the defaults from the glade file if we don't have
+			/* Use the defaults from the ui file if we don't have
 			 * good w, h geometry.
 			 */
 			gtk_window_resize (GTK_WINDOW (priv->dialog), w, h);
@@ -1530,7 +1534,7 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 	}
 
 	child = GTK_WIDGET (chat);
-	label = chat_window_create_label (window, chat); 
+	label = chat_window_create_label (window, chat);
 	gtk_widget_show (child);
 
 	g_signal_connect (chat, "notify::name",
@@ -1548,7 +1552,7 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 	gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK (priv->notebook), child, TRUE);
 	gtk_notebook_set_tab_detachable (GTK_NOTEBOOK (priv->notebook), child, TRUE);
 	gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (priv->notebook), child,
-					    TRUE, TRUE, GTK_PACK_START); 
+					    TRUE, TRUE, GTK_PACK_START);
 
 	DEBUG ("Chat added (%d references)", G_OBJECT (chat)->ref_count);
 }
@@ -1704,20 +1708,6 @@ empathy_chat_window_present_chat (EmpathyChat *chat)
 	empathy_chat_window_switch_to_chat (window, chat);
 	empathy_window_present (GTK_WINDOW (priv->dialog), TRUE);
 
- 	gtk_widget_grab_focus (chat->input_text_view); 
+ 	gtk_widget_grab_focus (chat->input_text_view);
 }
 
-#if 0
-static gboolean
-chat_window_should_play_sound (EmpathyChatWindow *window)
-{
-	EmpathyChatWindowPriv *priv = GET_PRIV (window);
-	gboolean               has_focus = FALSE;
-
-	g_return_val_if_fail (EMPATHY_IS_CHAT_WINDOW (window), FALSE);
-
-	g_object_get (priv->dialog, "has-toplevel-focus", &has_focus, NULL);
-
-	return !has_focus;
-}
-#endif
